#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include "conf.h"

struct odpath_t *load_conf(const char *file, struct odpath_t *root) {
char *buffer = malloc(2048), *section = malloc(16384);
struct odpath_t *path = root, *current = NULL;
FILE *fp;
int line = 0, sec_size = 0;

	if (path) while (path->next) path = path->next;

	if (!(fp = fopen(file, "r"))) {
		free(buffer);
		free(section);
		return root;
	}

	while (fgets(buffer, 2048, fp)) {
	int line_size = strlen(buffer);

		++line;
		/* empty lines and comments are ignored */
		if (!strchr("#\n\r", buffer[0]) && line_size) {
			/* continuation... */
			if (isspace(buffer[0])) {
			char *sol = buffer;

				if (!current) {
					error("%s:%i: data outside section", file, line);
					continue;
				}
				if (sec_size + line_size >= MAX_SECTION_SIZE)
					fatal("section is greater than MAX_SECTION_SIZE (%i)", MAX_SECTION_SIZE);
				/* strip leading spaces */
				while (isspace(*sol)) ++sol;
				strcat(section, sol);
			} else {
			char *tok;
			char *paths;

				/* new section... */
				if (current) {
					current->content = strdup(section);
					if (path) {
						path->next = current;
						path = current;
					} else {
						path = current;
						if (!root) root = path;
					}
				}
				if (!(current = calloc(1, sizeof(struct odpath_t))))
					fatal("malloc failed");

				memset(section, 0, MAX_SECTION_SIZE);

				/* read stuff */
				if (!(tok = strtok(buffer, " \t")) || (strcmp(tok, "enter") && strcmp(tok, "leave")))
					fatal("%s:%i: expected enter or leave", file, line);

				if (!strcmp(tok, "enter"))
					current->type = PT_ENTER;
				else
				if (!strcmp(tok, "leave"))
					current->type = PT_LEAVE;

				tok = strtok(NULL, "\n");
				paths = malloc(strlen(tok) + 3);
				strcpy(paths, ":");
				strcat(paths, tok);
				strcat(paths, ":");
				current->paths = paths;
			}
		}
	}
	if (current) {
		current->content = strdup(section);
		if (path) {
			path->next = current;
			path = current;
		} else {
			path = current;
			if (!root) root = path;
		}
	}
	fclose(fp);
	free(section);
	free(buffer);
	return root;
}

void fatal(const char *fmt, ...) {
va_list args;

	fprintf(stderr, "fatal: ");
	va_start(args, fmt);
	vfprintf(stderr, fmt, args);
	fprintf(stderr, "\n");
	va_end(args);
	exit(1);
}

void error(const char *fmt, ...) {
va_list args;

	fprintf(stderr, "error: ");
	va_start(args, fmt);
	vfprintf(stderr, fmt, args);
	fprintf(stderr, "\n");
	va_end(args);
}

void warning(const char *fmt, ...) {
va_list args;

	printf("warning: ");
	va_start(args, fmt);
	vprintf(fmt, args);
	va_end(args);
}

void info(const char *fmt, ...) {
va_list args;

	printf("ondir: ");
	va_start(args, fmt);
	vprintf(fmt, args);
	va_end(args);
}

